import { SCALEType } from '../../../scale/type/SCALEType';
export declare enum SubstrateStorageEntryHasher {
    BLAKE2_128 = 0,
    BLAKE2_256 = 1,
    BLAKE2_128_CONCAT = 2,
    TWOX128 = 3,
    TWOX256 = 4,
    TWOX64_CONCAT = 5,
    IDENTITY = 6
}
export declare abstract class SubstrateStorageEntry {
    readonly moduleName: string;
    readonly prefix: string;
    private readonly storageHash;
    constructor(moduleName: string, prefix: string);
    hash(...args: SCALEType[]): Promise<string>;
    protected abstract argsToKeys(args: SCALEType[]): Promise<string>;
    protected abstract hashArgs(args: SCALEType[]): Promise<string>;
    private hashPrefixTrie;
}
export declare class SubstratePlainStorageEntry extends SubstrateStorageEntry {
    protected argsToKeys(args: SCALEType[]): Promise<string>;
    protected hashArgs(args: SCALEType[]): Promise<string>;
}
export declare class SubstrateMapStorageEntry extends SubstrateStorageEntry {
    readonly hasher: SubstrateStorageEntryHasher;
    constructor(module: string, prefix: string, hasher: SubstrateStorageEntryHasher);
    protected argsToKeys(args: SCALEType[]): Promise<string>;
    protected hashArgs(args: SCALEType[]): Promise<string>;
}
export declare class SubstrateDoubleMapStorageEntry extends SubstrateStorageEntry {
    readonly firstHasher: SubstrateStorageEntryHasher;
    readonly secondHasher: SubstrateStorageEntryHasher;
    constructor(module: string, prefix: string, firstHasher: SubstrateStorageEntryHasher, secondHasher: SubstrateStorageEntryHasher);
    protected argsToKeys(args: SCALEType[]): Promise<string>;
    protected hashArgs(args: SCALEType[]): Promise<string>;
}
