"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var SCALEDecoder_1 = require("../../../../scale/SCALEDecoder");
var SCALEClass_1 = require("../../../../scale/type/SCALEClass");
var SCALEEnum_1 = require("../../../../scale/type/SCALEEnum");
var SubstrateStorageEntry_1 = require("../../../decorator/storage/SubstrateStorageEntry");
var StorageEntryType;
(function (StorageEntryType) {
    StorageEntryType[StorageEntryType["Plain"] = 0] = "Plain";
    StorageEntryType[StorageEntryType["Map"] = 1] = "Map";
    StorageEntryType[StorageEntryType["DoubleMap"] = 2] = "DoubleMap";
})(StorageEntryType || (StorageEntryType = {}));
var MetadataV11StorageEntryType = /** @class */ (function (_super) {
    __extends(MetadataV11StorageEntryType, _super);
    function MetadataV11StorageEntryType() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MetadataV11StorageEntryType.decode = function (network, raw) {
        var prefix = parseInt(raw.substr(0, 2), 16);
        var decoderMethod;
        switch (prefix) {
            case 0:
                decoderMethod = MetadataV11StorageEntryPlain.decode;
                break;
            case 1:
                decoderMethod = MetadataV11StorageEntryMap.decode;
                break;
            case 2:
                decoderMethod = MetadataV11StorageEntryDoubleMap.decode;
                break;
            default:
                throw new Error('Unkown metadata storage entry type');
        }
        var decoded = decoderMethod(network, raw.slice(2));
        return {
            bytesDecoded: 1 + decoded.bytesDecoded,
            decoded: decoded.decoded
        };
    };
    Object.defineProperty(MetadataV11StorageEntryType.prototype, "scaleFields", {
        get: function () {
            return __spreadArrays([this.type], this._scaleFields);
        },
        enumerable: true,
        configurable: true
    });
    return MetadataV11StorageEntryType;
}(SCALEClass_1.SCALEClass));
exports.MetadataV11StorageEntryType = MetadataV11StorageEntryType;
var MetadataV11StorageEntryPlain = /** @class */ (function (_super) {
    __extends(MetadataV11StorageEntryPlain, _super);
    function MetadataV11StorageEntryPlain(name) {
        var _this = _super.call(this) || this;
        _this.name = name;
        _this.type = SCALEEnum_1.SCALEEnum.from(StorageEntryType.Plain);
        _this._scaleFields = [_this.name];
        return _this;
    }
    MetadataV11StorageEntryPlain.decode = function (network, raw) {
        var decoder = new SCALEDecoder_1.SCALEDecoder(network, raw);
        var name = decoder.decodeNextString();
        return {
            bytesDecoded: name.bytesDecoded,
            decoded: new MetadataV11StorageEntryPlain(name.decoded)
        };
    };
    MetadataV11StorageEntryPlain.prototype.decorate = function (moduleName, prefix) {
        return new SubstrateStorageEntry_1.SubstratePlainStorageEntry(moduleName, prefix);
    };
    return MetadataV11StorageEntryPlain;
}(MetadataV11StorageEntryType));
exports.MetadataV11StorageEntryPlain = MetadataV11StorageEntryPlain;
var MetadataV11StorageEntryMap = /** @class */ (function (_super) {
    __extends(MetadataV11StorageEntryMap, _super);
    function MetadataV11StorageEntryMap(hasher, key, value, unused) {
        var _this = _super.call(this) || this;
        _this.hasher = hasher;
        _this.key = key;
        _this.value = value;
        _this.unused = unused;
        _this.type = SCALEEnum_1.SCALEEnum.from(StorageEntryType.Map);
        _this._scaleFields = [_this.hasher, _this.key, _this.value, _this.unused];
        return _this;
    }
    MetadataV11StorageEntryMap.decode = function (network, raw) {
        var decoder = new SCALEDecoder_1.SCALEDecoder(network, raw);
        var hasher = decoder.decodeNextEnum(function (value) { return SubstrateStorageEntry_1.SubstrateStorageEntryHasher[SubstrateStorageEntry_1.SubstrateStorageEntryHasher[value]]; });
        var key = decoder.decodeNextString();
        var value = decoder.decodeNextString();
        var unused = decoder.decodeNextBoolean();
        return {
            bytesDecoded: hasher.bytesDecoded + key.bytesDecoded + value.bytesDecoded + unused.bytesDecoded,
            decoded: new MetadataV11StorageEntryMap(hasher.decoded, key.decoded, value.decoded, unused.decoded)
        };
    };
    MetadataV11StorageEntryMap.prototype.decorate = function (moduleName, prefix) {
        return new SubstrateStorageEntry_1.SubstrateMapStorageEntry(moduleName, prefix, this.hasher.value);
    };
    return MetadataV11StorageEntryMap;
}(MetadataV11StorageEntryType));
exports.MetadataV11StorageEntryMap = MetadataV11StorageEntryMap;
var MetadataV11StorageEntryDoubleMap = /** @class */ (function (_super) {
    __extends(MetadataV11StorageEntryDoubleMap, _super);
    function MetadataV11StorageEntryDoubleMap(hasher1, key1, key2, value, hasher2) {
        var _this = _super.call(this) || this;
        _this.hasher1 = hasher1;
        _this.key1 = key1;
        _this.key2 = key2;
        _this.value = value;
        _this.hasher2 = hasher2;
        _this.type = SCALEEnum_1.SCALEEnum.from(StorageEntryType.Map);
        _this._scaleFields = [_this.hasher1, _this.key1, _this.key2, _this.value, _this.hasher2];
        return _this;
    }
    MetadataV11StorageEntryDoubleMap.decode = function (network, raw) {
        var decoder = new SCALEDecoder_1.SCALEDecoder(network, raw);
        var hasher1 = decoder.decodeNextEnum(function (value) { return SubstrateStorageEntry_1.SubstrateStorageEntryHasher[SubstrateStorageEntry_1.SubstrateStorageEntryHasher[value]]; });
        var key1 = decoder.decodeNextString();
        var key2 = decoder.decodeNextString();
        var value = decoder.decodeNextString();
        var hasher2 = decoder.decodeNextEnum(function (value) { return SubstrateStorageEntry_1.SubstrateStorageEntryHasher[SubstrateStorageEntry_1.SubstrateStorageEntryHasher[value]]; });
        return {
            bytesDecoded: hasher1.bytesDecoded + key1.bytesDecoded + key2.bytesDecoded + value.bytesDecoded + hasher2.bytesDecoded,
            decoded: new MetadataV11StorageEntryDoubleMap(hasher1.decoded, key1.decoded, key2.decoded, value.decoded, hasher2.decoded)
        };
    };
    MetadataV11StorageEntryDoubleMap.prototype.decorate = function (moduleName, prefix) {
        return new SubstrateStorageEntry_1.SubstrateDoubleMapStorageEntry(moduleName, prefix, this.hasher1.value, this.hasher2.value);
    };
    return MetadataV11StorageEntryDoubleMap;
}(MetadataV11StorageEntryType));
exports.MetadataV11StorageEntryDoubleMap = MetadataV11StorageEntryDoubleMap;
//# sourceMappingURL=MetadataV11StorageEntryType.js.map