"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var hex_1 = require("../../../../../../utils/hex");
var SCALEBytes_1 = require("./SCALEBytes");
var SCALEType_1 = require("./SCALEType");
var SCALEDataType;
(function (SCALEDataType) {
    SCALEDataType[SCALEDataType["None"] = 0] = "None";
    SCALEDataType[SCALEDataType["Raw"] = 1] = "Raw";
    SCALEDataType[SCALEDataType["BlakeTwo256"] = 2] = "BlakeTwo256";
    SCALEDataType[SCALEDataType["Sha256"] = 3] = "Sha256";
    SCALEDataType[SCALEDataType["Keccak256"] = 4] = "Keccak256";
    SCALEDataType[SCALEDataType["ShaThree256"] = 5] = "ShaThree256";
})(SCALEDataType = exports.SCALEDataType || (exports.SCALEDataType = {}));
var SCALEData = /** @class */ (function (_super) {
    __extends(SCALEData, _super);
    function SCALEData(type, bytes) {
        var _this = _super.call(this) || this;
        _this.type = type;
        _this.bytes = bytes;
        return _this;
    }
    SCALEData.from = function (type, bytes) {
        var dataType = type;
        var buffer;
        if (typeof bytes === 'string' && hex_1.isHex(bytes)) {
            buffer = Buffer.from(hex_1.stripHexPrefix(bytes), 'hex');
        }
        else if (bytes && !(typeof bytes === 'string')) {
            buffer = Buffer.from(bytes);
        }
        else {
            dataType = SCALEDataType.None;
            buffer = null;
        }
        return new SCALEData(dataType, buffer);
    };
    SCALEData.decode = function (hex) {
        var encoded = hex_1.hexToBytes(hex);
        var indicator = encoded[0];
        var bytesDecoded;
        var type;
        if (indicator === 0) {
            bytesDecoded = 1;
            type = SCALEDataType.None;
        }
        else if (indicator >= 1 && indicator <= 33) {
            bytesDecoded = indicator;
            type = SCALEDataType.Raw;
        }
        else if (indicator >= 34 && indicator <= 37) {
            bytesDecoded = 32 + 1;
            type = SCALEDataType[SCALEDataType[indicator - 32]];
        }
        else {
            throw new Error('SCALEData#decode: Unknown data type');
        }
        var buffer = type !== SCALEDataType.None
            ? SCALEBytes_1.SCALEBytes.from(encoded.subarray(1, bytesDecoded)).bytes
            : null;
        return {
            bytesDecoded: bytesDecoded,
            decoded: new SCALEData(type, buffer)
        };
    };
    SCALEData.prototype.toString = function () {
        if (this.type !== SCALEDataType.None && this.bytes === null) {
            throw new Error("SCALEData#toString: type is " + SCALEDataType[this.type] + " but data is null.");
        }
        switch (this.type) {
            case SCALEDataType.None:
                return 'null';
            case SCALEDataType.Raw:
                var textDecoder = new TextDecoder();
                return textDecoder.decode(this.bytes);
            default:
                return ''; // TODO: add support for other types
        }
    };
    SCALEData.prototype._encode = function () {
        if (this.type !== SCALEDataType.None && this.bytes === null) {
            throw new Error("SCALEData#encode: type is " + SCALEDataType[this.type] + " but data is null.");
        }
        switch (this.type) {
            case SCALEDataType.None:
                return '0';
            case SCALEDataType.Raw:
                return SCALEBytes_1.SCALEBytes.from(this.bytes).encode();
            default:
                throw new Error("SCALEData#encode: type " + SCALEDataType[this.type] + " is not supported.");
        }
    };
    return SCALEData;
}(SCALEType_1.SCALEType));
exports.SCALEData = SCALEData;
//# sourceMappingURL=SCALEData.js.map