"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var bignumber_1 = require("../../../../../../dependencies/src/bignumber.js-9.0.0/bignumber");
var hex_1 = require("../../../../../../utils/hex");
var SCALEType_1 = require("./SCALEType");
var IMMORTAL_ENCODED = '00';
var ERA_DEFAULT_PERIOD = 50; // 5 min at 6s block times
var SCALEEra = /** @class */ (function (_super) {
    __extends(SCALEEra, _super);
    function SCALEEra(period, phase) {
        var _this = _super.call(this) || this;
        _this.period = period;
        _this.phase = phase;
        return _this;
    }
    SCALEEra.Immortal = function () {
        return new SCALEEra(0, 0);
    };
    SCALEEra.Mortal = function (config) {
        var chainHeight = bignumber_1.default.isBigNumber(config.chainHeight) ? config.chainHeight.toNumber() : config.chainHeight;
        var period = bignumber_1.default.isBigNumber(config.period) ? config.period.toNumber() : config.period;
        period = Math.pow(2, Math.ceil(Math.log2(period || ERA_DEFAULT_PERIOD)));
        period = Math.min(Math.max(period, 4), 1 << 16);
        var phase = chainHeight % period;
        var quantizeFactor = Math.max(period >> 12, 1);
        var quantizePhase = (phase / quantizeFactor) * quantizeFactor;
        return new SCALEEra(period, quantizePhase);
    };
    Object.defineProperty(SCALEEra.prototype, "isMortal", {
        get: function () {
            return this.period !== 0;
        },
        enumerable: true,
        configurable: true
    });
    SCALEEra.decode = function (hex) {
        var _hex = hex_1.stripHexPrefix(hex);
        return _hex.substr(0, 2) === IMMORTAL_ENCODED ? SCALEEra.decodeImmortal() : SCALEEra.decodeMortal(_hex);
    };
    SCALEEra.decodeImmortal = function () {
        return {
            bytesDecoded: 1,
            decoded: SCALEEra.Immortal()
        };
    };
    SCALEEra.decodeMortal = function (hex) {
        var encoded = parseInt(hex_1.changeEndianness(hex.substr(0, 4)), 16);
        var period = 2 << encoded % (1 << 4);
        var quantizeFactor = Math.max(period >> 12, 1);
        var phase = (encoded >> 4) * quantizeFactor;
        if (period < 4 || period < phase) {
            throw new Error('SCALEEra#decodeMortal: Invalid mortal era');
        }
        return {
            bytesDecoded: 2,
            decoded: new SCALEEra(period, phase)
        };
    };
    SCALEEra.prototype.toString = function () {
        return JSON.stringify({
            isMortal: this.isMortal,
            period: this.period,
            phase: this.phase
        }, null, 2);
    };
    SCALEEra.prototype._encode = function () {
        if (!this.isMortal) {
            return IMMORTAL_ENCODED;
        }
        var quantizeFactor = Math.max(this.period >> 12, 1);
        var trailingZeros = this.getTrailingZeros(this.period);
        var encoded = Math.min(15, Math.max(1, trailingZeros - 1)) + ((this.phase / quantizeFactor) << 4);
        return hex_1.changeEndianness(hex_1.toHexStringRaw(encoded, 16));
    };
    SCALEEra.prototype.getTrailingZeros = function (value) {
        return value.toString(2).split('').reverse().join('').indexOf('1');
    };
    return SCALEEra;
}(SCALEType_1.SCALEType));
exports.SCALEEra = SCALEEra;
//# sourceMappingURL=SCALEEra.js.map