import { SubstrateTransactionType } from '../data/transaction/SubstrateTransaction';
export declare const supportedRpcMethods: {
    author: readonly ["submitExtrinsic"];
    chain: readonly ["getBlock", "getBlockHash"];
    state: readonly ["getMetadata", "getStorage", "getRuntimeVersion"];
    payment: readonly ["queryInfo"];
};
export declare const supportedStorageEntries: {
    Identity: readonly ["IdentityOf", "SuperOf", "SubsOf"];
    Staking: readonly ["Bonded", "Ledger", "Payee", "Nominators", "CurrentEra", "ActiveEra", "EraElectionStatus", "ErasStakers", "ErasStakersClipped", "ErasValidatorPrefs", "ErasValidatorReward", "ErasRewardPoints", "SlashingSpans"];
    Session: readonly ["Validators"];
    System: readonly ["Account"];
};
export declare const supportedCalls: {
    Balances: readonly ["transfer"];
    Staking: readonly ["bond", "bond_extra", "unbond", "withdraw_unbonded", "nominate", "chill", "set_payee", "set_controller", "payout_stakers", "rebond"];
    Utility: readonly ["batch"];
};
export declare const supportedConstants: {
    Babe: readonly ["EpochDuration", "ExpectedBlockTime"];
    Balances: readonly ["ExistentialDeposit"];
    Staking: readonly ["SessionsPerEra"];
};
export declare type SubstrateRpcModuleName = keyof typeof supportedRpcMethods;
export declare type SubstrateRpcMethodName<T extends SubstrateRpcModuleName> = {
    [S in T]: typeof supportedRpcMethods[S][number];
}[T];
export declare type SubstrateStorageModuleName = keyof typeof supportedStorageEntries;
export declare type SubstrateStorageEntryName<T extends SubstrateStorageModuleName> = {
    [S in T]: typeof supportedStorageEntries[S][number];
}[T];
export declare type SubstrateCallModuleName = keyof typeof supportedCalls;
export declare type SubstrateCallName<T extends SubstrateCallModuleName> = {
    [S in T]: typeof supportedCalls[S][number];
}[T];
export declare type SubstrateConstantModuleName = keyof typeof supportedConstants;
export declare type SubstrateConstantName<T extends SubstrateConstantModuleName> = {
    [S in T]: typeof supportedConstants[S][number];
}[T];
export declare const supportedCallEndpoints: Map<SubstrateTransactionType, [SubstrateCallModuleName, SubstrateCallName<any>]>;
