"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var SubstrateTransaction_1 = require("../data/transaction/SubstrateTransaction");
exports.supportedRpcMethods = {
    author: ['submitExtrinsic'],
    chain: ['getBlock', 'getBlockHash'],
    state: ['getMetadata', 'getStorage', 'getRuntimeVersion'],
    payment: ['queryInfo']
};
// to keep the generated API as minimal required
exports.supportedStorageEntries = {
    Identity: ['IdentityOf', 'SuperOf', 'SubsOf'],
    Staking: [
        'Bonded',
        'Ledger',
        'Payee',
        'Nominators',
        'CurrentEra',
        'ActiveEra',
        'EraElectionStatus',
        'ErasStakers',
        'ErasStakersClipped',
        'ErasValidatorPrefs',
        'ErasValidatorReward',
        'ErasRewardPoints',
        'SlashingSpans'
    ],
    Session: ['Validators'],
    System: ['Account']
};
exports.supportedCalls = {
    Balances: ['transfer'],
    Staking: [
        'bond',
        'bond_extra',
        'unbond',
        'withdraw_unbonded',
        'nominate',
        'chill',
        'set_payee',
        'set_controller',
        'payout_stakers',
        'rebond'
    ],
    Utility: ['batch']
};
exports.supportedConstants = {
    Babe: ['EpochDuration', 'ExpectedBlockTime'],
    Balances: ['ExistentialDeposit'],
    Staking: ['SessionsPerEra']
};
exports.supportedCallEndpoints = new Map([
    createCallEndpointEntry(SubstrateTransaction_1.SubstrateTransactionType.TRANSFER, 'Balances', 'transfer'),
    createCallEndpointEntry(SubstrateTransaction_1.SubstrateTransactionType.BOND, 'Staking', 'bond'),
    createCallEndpointEntry(SubstrateTransaction_1.SubstrateTransactionType.UNBOND, 'Staking', 'unbond'),
    createCallEndpointEntry(SubstrateTransaction_1.SubstrateTransactionType.BOND_EXTRA, 'Staking', 'bond_extra'),
    createCallEndpointEntry(SubstrateTransaction_1.SubstrateTransactionType.WITHDRAW_UNBONDED, 'Staking', 'withdraw_unbonded'),
    createCallEndpointEntry(SubstrateTransaction_1.SubstrateTransactionType.NOMINATE, 'Staking', 'nominate'),
    createCallEndpointEntry(SubstrateTransaction_1.SubstrateTransactionType.CANCEL_NOMINATION, 'Staking', 'chill'),
    createCallEndpointEntry(SubstrateTransaction_1.SubstrateTransactionType.COLLECT_PAYOUT, 'Staking', 'payout_stakers'),
    createCallEndpointEntry(SubstrateTransaction_1.SubstrateTransactionType.SET_PAYEE, 'Staking', 'set_payee'),
    createCallEndpointEntry(SubstrateTransaction_1.SubstrateTransactionType.SET_CONTROLLER, 'Staking', 'set_controller'),
    createCallEndpointEntry(SubstrateTransaction_1.SubstrateTransactionType.REBOND, 'Staking', 'rebond'),
    createCallEndpointEntry(SubstrateTransaction_1.SubstrateTransactionType.SUBMIT_BATCH, 'Utility', 'batch')
]);
function createCallEndpointEntry(transactionType, moduleName, callName) {
    return [transactionType, [moduleName, callName]];
}
//# sourceMappingURL=supported.js.map