"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var MichelsonList_1 = require("./generics/MichelsonList");
var MichelsonOption_1 = require("./generics/MichelsonOption");
var MichelsonOr_1 = require("./generics/MichelsonOr");
var MichelsonPair_1 = require("./generics/MichelsonPair");
var MichelsonAddress_1 = require("./primitives/MichelsonAddress");
var MichelsonBool_1 = require("./primitives/MichelsonBool");
var MichelsonBytes_1 = require("./primitives/MichelsonBytes");
var MichelsonInt_1 = require("./primitives/MichelsonInt");
var MichelsonString_1 = require("./primitives/MichelsonString");
var MichelsonUnit_1 = require("./primitives/MichelsonUnit");
exports.michelsonTypeFactories = {
    nat: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonInt_1.MichelsonInt.from(args[0]);
    },
    int: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonInt_1.MichelsonInt.from(args[0]);
    },
    string: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonString_1.MichelsonString.from(args[0]);
    },
    bytes: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonBytes_1.MichelsonBytes.from(args[0]);
    },
    mutez: function () { return notSupported('mutez'); },
    bool: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonBool_1.MichelsonBool.from(args[0]);
    },
    key_hash: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonString_1.MichelsonString.from(args[0]);
    },
    timestamp: function () { return notSupported('timestamp'); },
    address: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonAddress_1.MichelsonAddress.from(args[0]);
    },
    key: function () { return notSupported('key'); },
    unit: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonUnit_1.MichelsonUnit.from(args[0]);
    },
    signature: function () { return notSupported('signature'); },
    option: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonOption_1.MichelsonOption.from(args[0], args[1]);
    },
    list: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonList_1.MichelsonList.from(args[0], args[1]);
    },
    set: function () { return notSupported('set'); },
    operation: function () { return notSupported('operation'); },
    contract: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonAddress_1.MichelsonAddress.from(args[0]);
    },
    pair: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonPair_1.MichelsonPair.from(args[0], args[1], args[2]);
    },
    or: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonOr_1.MichelsonOr.from(args[0], args[1], args[2]);
    },
    lambda: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonString_1.MichelsonString.from(args[0]);
    },
    map: function () { return notSupported('map'); },
    big_map: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return MichelsonInt_1.MichelsonInt.from(args[0]);
    },
    chain_id: function () { return notSupported('chain_id'); },
};
function notSupported(type) {
    throw new Error("Michelson type " + type + " is not supported.");
}
//# sourceMappingURL=factories.js.map