"use strict";
// tslint:disable: max-classes-per-file
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Lazy_1 = require("../../../../../data/Lazy");
var utils_1 = require("../../utils");
var MichelsonType_1 = require("../MichelsonType");
var MichelsonOption = /** @class */ (function (_super) {
    __extends(MichelsonOption, _super);
    function MichelsonOption() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MichelsonOption.from = function (value, mappingFunction, name) {
        if (value instanceof MichelsonOption) {
            return value;
        }
        if (!(value instanceof MichelsonType_1.MichelsonType) && typeof mappingFunction !== 'function') {
            throw new Error('MichelsonOption: unknown generic mapping factory function.');
        }
        return utils_1.isMichelinePrimitiveApplication(value)
            ? MichelsonOption.fromMicheline(value, mappingFunction, name)
            : MichelsonOption.fromUnknown(value, mappingFunction, name);
    };
    MichelsonOption.fromMicheline = function (micheline, mappingFunction, name) {
        return MichelsonOption.fromUnknown(micheline.prim === 'Some' && micheline.args ? micheline.args[0] : null, mappingFunction, name);
    };
    MichelsonOption.fromUnknown = function (unknownValue, mappingFunction, name) {
        if (unknownValue === undefined || unknownValue === null) {
            return new MichelsonNone(name);
        }
        var lazyValue = unknownValue instanceof MichelsonType_1.MichelsonType
            ? new Lazy_1.Lazy(function () { return unknownValue; })
            : new Lazy_1.Lazy(function () {
                var value = typeof mappingFunction === 'function' ? mappingFunction(unknownValue) : undefined;
                if (!(value instanceof MichelsonType_1.MichelsonType)) {
                    throw new Error('MichelsonOption: unknown generic mapping type.');
                }
                return value;
            });
        return new MichelsonSome(lazyValue, name);
    };
    return MichelsonOption;
}(MichelsonType_1.MichelsonType));
exports.MichelsonOption = MichelsonOption;
var MichelsonSome = /** @class */ (function (_super) {
    __extends(MichelsonSome, _super);
    function MichelsonSome(value, name) {
        var _this = _super.call(this, name) || this;
        _this.value = value;
        _this.type = 'Some';
        return _this;
    }
    MichelsonSome.prototype.asRawValue = function () {
        var _a;
        var value = this.value.get().asRawValue();
        return this.name ? (_a = {}, _a[this.name] = value, _a) : value;
    };
    MichelsonSome.prototype.toMichelineJSON = function () {
        return {
            prim: 'Some',
            args: [this.value.get().toMichelineJSON()]
        };
    };
    MichelsonSome.prototype.eval = function () {
        this.value.get();
    };
    return MichelsonSome;
}(MichelsonOption));
exports.MichelsonSome = MichelsonSome;
var MichelsonNone = /** @class */ (function (_super) {
    __extends(MichelsonNone, _super);
    function MichelsonNone() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.type = 'None';
        return _this;
    }
    MichelsonNone.prototype.asRawValue = function () {
        var _a;
        return this.name ? (_a = {}, _a[this.name] = null, _a) : null;
    };
    MichelsonNone.prototype.toMichelineJSON = function () {
        return {
            prim: 'None'
        };
    };
    return MichelsonNone;
}(MichelsonOption));
exports.MichelsonNone = MichelsonNone;
//# sourceMappingURL=MichelsonOption.js.map