"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable: max-classes-per-file
var Lazy_1 = require("../../../../../data/Lazy");
var error_1 = require("../../../../../utils/error");
var utils_1 = require("../../utils");
var MichelsonType_1 = require("../MichelsonType");
var MichelsonOr = /** @class */ (function (_super) {
    __extends(MichelsonOr, _super);
    function MichelsonOr(value, name) {
        var _this = _super.call(this, name) || this;
        _this.value = value;
        return _this;
    }
    MichelsonOr.from = function (or, firstMappingFunction, secondMappingFunction, name) {
        if (or instanceof MichelsonOr) {
            return or;
        }
        if (!(or instanceof MichelsonType_1.MichelsonType) && typeof firstMappingFunction !== 'function' || typeof secondMappingFunction !== 'function') {
            throw new Error('MichelsonPair: unknown generic mapping factory functions.');
        }
        return utils_1.isMichelinePrimitiveApplication(or)
            ? MichelsonOr.fromMicheline(or, firstMappingFunction, secondMappingFunction, name)
            : MichelsonOr.fromUnknown(or, firstMappingFunction, secondMappingFunction, name);
    };
    MichelsonOr.fromMicheline = function (micheline, firstMappingFunction, secondMappingFunction, name) {
        if (!MichelsonOr.isOr(micheline)) {
            throw error_1.invalidArgumentTypeError('MichelsonOr', 'prim: Left | Right', "prim: " + micheline.prim);
        }
        if (micheline.args === undefined) {
            throw error_1.invalidArgumentTypeError('MichelsonOr', 'args: <array>', 'args: undefined');
        }
        return MichelsonOr.fromUnknown([micheline.prim, micheline.args[0]], firstMappingFunction, secondMappingFunction, name);
    };
    MichelsonOr.fromUnknown = function (unknownValue, firstMappingFunction, secondMappingFunction, name) {
        if ((!Array.isArray(unknownValue) || unknownValue.length !== 2 || typeof unknownValue[0] !== 'string')) {
            throw error_1.invalidArgumentTypeError('MichelsonOr', "MichelsonOr or tuple<'Left' | 'Right', any>", typeof unknownValue + ": " + unknownValue);
        }
        var type = unknownValue[0];
        if (type.toLowerCase() === 'left' || type.toLowerCase() === 'l') {
            return MichelsonOr.create('Left', unknownValue[1], firstMappingFunction, name);
        }
        else if (type.toLowerCase() === 'right' || type.toLowerCase() === 'r') {
            return MichelsonOr.create('Right', unknownValue[1], secondMappingFunction, name);
        }
        else {
            throw new Error("MichelsonOr: unknown type " + unknownValue[0] + ", expected 'Left' or 'Right'.");
        }
    };
    MichelsonOr.isOr = function (unknownValue) {
        return (unknownValue instanceof MichelsonOr ||
            (utils_1.isMichelinePrimitiveApplication(unknownValue) &&
                (unknownValue.prim === 'Left' || unknownValue.prim === 'Right')));
    };
    MichelsonOr.create = function (type, value, mappingFunction, name) {
        var lazyValue = value instanceof MichelsonType_1.MichelsonType
            ? new Lazy_1.Lazy(function () { return value; })
            : new Lazy_1.Lazy(function () {
                var mappedValue = typeof mappingFunction === 'function' ? mappingFunction(value) : undefined;
                if (!(mappedValue instanceof MichelsonType_1.MichelsonType)) {
                    throw new Error('MichelsonOr: unknown generic mapping type.');
                }
                return mappedValue;
            });
        return type === 'Left' ? new MichelsonLeft(lazyValue, name) : new MichelsonRight(lazyValue, name);
    };
    MichelsonOr.prototype.asRawValue = function () {
        var _a;
        return this.name ? (_a = {}, _a[this.name] = this.value.get().asRawValue(), _a) : this.value.get().asRawValue();
    };
    MichelsonOr.prototype.toMichelineJSON = function () {
        return {
            prim: this.type,
            args: [
                this.value.get().toMichelineJSON()
            ]
        };
    };
    MichelsonOr.prototype.eval = function () {
        this.value.get();
    };
    return MichelsonOr;
}(MichelsonType_1.MichelsonType));
exports.MichelsonOr = MichelsonOr;
var MichelsonLeft = /** @class */ (function (_super) {
    __extends(MichelsonLeft, _super);
    function MichelsonLeft() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.type = 'Left';
        return _this;
    }
    return MichelsonLeft;
}(MichelsonOr));
exports.MichelsonLeft = MichelsonLeft;
var MichelsonRight = /** @class */ (function (_super) {
    __extends(MichelsonRight, _super);
    function MichelsonRight() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.type = 'Right';
        return _this;
    }
    return MichelsonRight;
}(MichelsonOr));
exports.MichelsonRight = MichelsonRight;
//# sourceMappingURL=MichelsonOr.js.map