"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Lazy_1 = require("../../../../../data/Lazy");
var error_1 = require("../../../../../utils/error");
var type_1 = require("../../../../../utils/type");
var utils_1 = require("../../utils");
var MichelsonType_1 = require("../MichelsonType");
var MichelsonPair = /** @class */ (function (_super) {
    __extends(MichelsonPair, _super);
    function MichelsonPair(first, second, name) {
        var _this = _super.call(this, name) || this;
        _this.first = first;
        _this.second = second;
        return _this;
    }
    MichelsonPair.from = function (pair, firstMappingFunction, secondMappingFunction, name) {
        if (pair instanceof MichelsonPair) {
            return pair;
        }
        return utils_1.isMichelinePrimitiveApplication(pair)
            ? MichelsonPair.fromMicheline(pair, firstMappingFunction, secondMappingFunction, name)
            : MichelsonPair.fromUnknown(pair, firstMappingFunction, secondMappingFunction, name);
    };
    MichelsonPair.fromMicheline = function (micheline, firstMappingFunction, secondMappingFunction, name) {
        if (micheline.prim !== 'Pair') {
            throw error_1.invalidArgumentTypeError('MichelsonPair', 'prim: Pair', "prim: " + micheline.prim);
        }
        if (micheline.args === undefined || micheline.args.length !== 2) {
            throw error_1.invalidArgumentTypeError('MichelsonPair', 'args: <tuple>', 'args: undefined | <array>');
        }
        return MichelsonPair.fromUnknown(micheline.args, firstMappingFunction, secondMappingFunction, name);
    };
    MichelsonPair.fromUnknown = function (unknownValue, firstMappingFunction, secondMappingFunction, name) {
        if (!(unknownValue instanceof Object) && (!Array.isArray(unknownValue) || unknownValue.length !== 2)) {
            throw error_1.invalidArgumentTypeError('MichelsonPair', 'tuple or object', typeof unknownValue + ": " + unknownValue);
        }
        var _a = Array.isArray(unknownValue)
            ? [MichelsonPair.asRawValue(unknownValue[0], firstMappingFunction), MichelsonPair.asRawValue(unknownValue[1], secondMappingFunction)]
            : [MichelsonPair.asRawValue(unknownValue, firstMappingFunction), MichelsonPair.asRawValue(unknownValue, secondMappingFunction)], first = _a[0], second = _a[1];
        return new MichelsonPair(first, second, name);
    };
    MichelsonPair.asRawValue = function (unknownValue, mappingFactory) {
        if (!(unknownValue instanceof MichelsonType_1.MichelsonType) && typeof mappingFactory !== 'function') {
            throw new Error('MichelsonPair: unknown generic mapping factory function.');
        }
        return unknownValue instanceof MichelsonType_1.MichelsonType
            ? new Lazy_1.Lazy(function () { return unknownValue; })
            : new Lazy_1.Lazy(function () {
                var value = typeof mappingFactory === 'function' ? mappingFactory(unknownValue) : undefined;
                if (!(value instanceof MichelsonType_1.MichelsonType)) {
                    throw new Error('MichelsonPair: unknown generic mapping type.');
                }
                return value;
            });
    };
    MichelsonPair.prototype.asRawValue = function () {
        var _a;
        var first = this.first.get().asRawValue();
        var second = this.second.get().asRawValue();
        var value = type_1.isRecord(first) && type_1.isRecord(second)
            ? Object.assign(first, second)
            : [first, second];
        return this.name ? (_a = {}, _a[this.name] = value, _a) : value;
    };
    MichelsonPair.prototype.toMichelineJSON = function () {
        return {
            prim: 'Pair',
            args: [
                this.first.get().toMichelineJSON(),
                this.second.get().toMichelineJSON()
            ]
        };
    };
    MichelsonPair.prototype.eval = function () {
        this.first.get();
        this.second.get();
    };
    return MichelsonPair;
}(MichelsonType_1.MichelsonType));
exports.MichelsonPair = MichelsonPair;
//# sourceMappingURL=MichelsonPair.js.map