"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var errors_1 = require("../errors");
var CosmosTransaction_1 = require("../protocols/cosmos/CosmosTransaction");
var ProtocolSymbols_1 = require("../utils/ProtocolSymbols");
var inter_app_communication_protocol_1 = require("./inter-app-communication-protocol");
var interfaces_1 = require("./interfaces");
var aeternity_transactions_validator_1 = require("./unsigned-transactions/aeternity-transactions.validator");
var bitcoin_transactions_validator_1 = require("./unsigned-transactions/bitcoin-transactions.validator");
var cosmos_transactions_validator_1 = require("./unsigned-transactions/cosmos-transactions.validator");
var ethereum_transactions_validator_1 = require("./unsigned-transactions/ethereum-transactions.validator");
var substrate_transactions_validator_1 = require("./unsigned-transactions/substrate-transactions.validator");
var tezos_transactions_validator_1 = require("./unsigned-transactions/tezos-transactions.validator");
var xtz_btc_transactions_validator_1 = require("./unsigned-transactions/xtz-btc-transactions.validator");
var accountShareResponse = require('./schemas/generated/account-share-response.json');
var messageSignRequest = require('./schemas/generated/message-sign-request.json');
var messageSignResponse = require('./schemas/generated/message-sign-response.json');
var unsignedTransactionAeternity = require('./schemas/generated/transaction-sign-request-aeternity.json');
var unsignedTransactionBitcoin = require('./schemas/generated/transaction-sign-request-bitcoin.json');
var unsignedTransactionCosmos = require('./schemas/generated/transaction-sign-request-cosmos.json');
var unsignedTransactionEthereum = require('./schemas/generated/transaction-sign-request-ethereum.json');
var unsignedTransactionTezos = require('./schemas/generated/transaction-sign-request-tezos.json');
var unsignedTransactionSubstrate = require('./schemas/generated/transaction-sign-request-substrate.json');
var signedTransactionAeternity = require('./schemas/generated/transaction-sign-response-aeternity.json');
var signedTransactionBitcoin = require('./schemas/generated/transaction-sign-response-bitcoin.json');
var signedTransactionCosmos = require('./schemas/generated/transaction-sign-response-cosmos.json');
var signedTransactionEthereum = require('./schemas/generated/transaction-sign-response-ethereum.json');
var signedTransactionTezos = require('./schemas/generated/transaction-sign-response-tezos.json');
var signedTransactionSubstrate = require('./schemas/generated/transaction-sign-response-substrate.json');
function unsignedTransactionTransformerCosmos(value) {
    value.transaction = CosmosTransaction_1.CosmosTransaction.fromJSON(value);
    return value;
}
var IACPayloadType;
(function (IACPayloadType) {
    IACPayloadType[IACPayloadType["FULL"] = 0] = "FULL";
    IACPayloadType[IACPayloadType["CHUNKED"] = 1] = "CHUNKED";
})(IACPayloadType = exports.IACPayloadType || (exports.IACPayloadType = {}));
var Serializer = /** @class */ (function () {
    function Serializer() {
    }
    Serializer.addSchema = function (schemaId, schema, protocol) {
        var protocolSpecificSchemaName = Serializer.getSchemaName(schemaId, protocol);
        if (this.schemas.has(protocolSpecificSchemaName)) {
            throw new errors_1.SerializerError(errors_1.SerializerErrorType.SCHEMA_ALREADY_EXISTS, "Schema " + protocolSpecificSchemaName + " already exists");
        }
        this.schemas.set(protocolSpecificSchemaName, schema);
    };
    Serializer.getSchema = function (schemaId, protocol) {
        var _a;
        var protocolSpecificSchemaName = Serializer.getSchemaName(schemaId, protocol);
        // Try to get the protocol specific scheme, if it doesn't exist fall back to the generic one
        var schema = (_a = this.schemas.get(protocolSpecificSchemaName), (_a !== null && _a !== void 0 ? _a : this.schemas.get(Serializer.getSchemaName(schemaId))));
        if (!schema) {
            throw new errors_1.SerializerError(errors_1.SerializerErrorType.SCHEMA_DOES_NOT_EXISTS, "Schema " + protocolSpecificSchemaName + " does not exist");
        }
        return schema;
    };
    Serializer.getSchemaName = function (schemaId, protocol) {
        var schemaName = schemaId + "-" + protocol;
        if (protocol !== undefined && schemaId === interfaces_1.IACMessageType.TransactionSignRequest || schemaId === interfaces_1.IACMessageType.TransactionSignResponse) {
            var split = schemaName.split('-');
            if (split.length >= 3 && split[1] + "-" + split[2] === ProtocolSymbols_1.SubProtocolSymbols.ETH_ERC20) {
                return schemaId + "-" + ProtocolSymbols_1.SubProtocolSymbols.ETH_ERC20;
            }
        }
        return protocol ? schemaId + "-" + protocol : schemaId.toString();
    };
    Serializer.prototype.serialize = function (messages, chunkSize) {
        if (chunkSize === void 0) { chunkSize = 0; }
        return __awaiter(this, void 0, void 0, function () {
            var iacps;
            return __generator(this, function (_a) {
                if (messages.every(function (message) {
                    return Serializer.getSchema(message.type, message.protocol);
                })) {
                    iacps = inter_app_communication_protocol_1.IACProtocol.fromDecoded(JSON.parse(JSON.stringify(messages)), chunkSize);
                    return [2 /*return*/, iacps.map(function (iac) { return iac.encoded(); })];
                }
                else {
                    throw new errors_1.SerializerError(errors_1.SerializerErrorType.SCHEMA_DOES_NOT_EXISTS, "Unknown schema");
                }
                return [2 /*return*/];
            });
        });
    };
    Serializer.prototype.deserialize = function (data) {
        return __awaiter(this, void 0, void 0, function () {
            var result, deserializedIACMessageDefinitionObjects;
            var _this = this;
            return __generator(this, function (_a) {
                result = inter_app_communication_protocol_1.IACProtocol.fromEncoded(data);
                deserializedIACMessageDefinitionObjects = result
                    .map(function (el) { return el.payload; })
                    .map(function (el) { return el.asJson(); })
                    .reduce(function (pv, cv) { return pv.concat.apply(pv, cv); }, []);
                return [2 /*return*/, Promise.all(deserializedIACMessageDefinitionObjects.map(function (object) {
                        var unsignedTx = object.payload;
                        var validator = _this.serializationValidatorByProtocolIdentifier(object.protocol);
                        return validator.validateUnsignedTransaction(unsignedTx);
                    })).then(function () {
                        return deserializedIACMessageDefinitionObjects;
                    })];
            });
        });
    };
    Serializer.prototype.serializationValidatorByProtocolIdentifier = function (protocolIdentifier) {
        var validators = {
            // TODO: Exhaustive list?
            eth: ethereum_transactions_validator_1.EthereumTransactionValidator,
            btc: bitcoin_transactions_validator_1.BitcoinTransactionValidator,
            grs: bitcoin_transactions_validator_1.BitcoinTransactionValidator,
            ae: aeternity_transactions_validator_1.AeternityTransactionValidator,
            xtz: tezos_transactions_validator_1.TezosTransactionValidator,
            cosmos: cosmos_transactions_validator_1.CosmosTransactionValidator,
            polkadot: substrate_transactions_validator_1.SubstrateTransactionValidator,
            kusama: substrate_transactions_validator_1.SubstrateTransactionValidator,
            'xtz-btc': xtz_btc_transactions_validator_1.TezosBTCTransactionValidator
        };
        var exactMatch = Object.keys(validators).find(function (protocol) { return protocolIdentifier === protocol; });
        var startsWith = Object.keys(validators).find(function (protocol) { return protocolIdentifier.startsWith(protocol); });
        var validator = exactMatch ? exactMatch : startsWith;
        // TODO: Only use validator if it's a transaction
        // if (!validator) {
        //   throw Error(`Validator not implemented for ${protocolIdentifier}, ${exactMatch}, ${startsWith}, ${validator}`)
        // }
        return new validators[(validator !== null && validator !== void 0 ? validator : 'eth')]();
    };
    Serializer.schemas = new Map();
    return Serializer;
}());
exports.Serializer = Serializer;
// Serializer.addSchema(IACMessageType.MetadataRequest, '')
// Serializer.addSchema(IACMessageType.MetadataResponse, '')
// Serializer.addSchema(IACMessageType.AccountShareRequest, accountShareRequest)
Serializer.addSchema(interfaces_1.IACMessageType.AccountShareResponse, { schema: accountShareResponse });
Serializer.addSchema(interfaces_1.IACMessageType.MessageSignRequest, { schema: messageSignRequest });
Serializer.addSchema(interfaces_1.IACMessageType.MessageSignResponse, { schema: messageSignResponse });
// TODO: Make sure that we have a schema for every protocol we support
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignRequest, { schema: unsignedTransactionAeternity }, ProtocolSymbols_1.MainProtocolSymbols.AE);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignRequest, { schema: unsignedTransactionBitcoin }, ProtocolSymbols_1.MainProtocolSymbols.BTC);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignRequest, { schema: unsignedTransactionBitcoin }, ProtocolSymbols_1.MainProtocolSymbols.GRS);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignRequest, { schema: unsignedTransactionCosmos, transformer: unsignedTransactionTransformerCosmos }, ProtocolSymbols_1.MainProtocolSymbols.COSMOS);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignRequest, { schema: unsignedTransactionEthereum }, ProtocolSymbols_1.MainProtocolSymbols.ETH);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignRequest, { schema: unsignedTransactionEthereum }, ProtocolSymbols_1.SubProtocolSymbols.ETH_ERC20);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignRequest, { schema: unsignedTransactionTezos }, ProtocolSymbols_1.MainProtocolSymbols.XTZ);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignRequest, { schema: unsignedTransactionTezos }, ProtocolSymbols_1.SubProtocolSymbols.XTZ_BTC);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignRequest, { schema: unsignedTransactionTezos }, ProtocolSymbols_1.SubProtocolSymbols.XTZ_KT);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignRequest, { schema: unsignedTransactionTezos }, ProtocolSymbols_1.SubProtocolSymbols.XTZ_USD);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignRequest, { schema: unsignedTransactionSubstrate }, ProtocolSymbols_1.MainProtocolSymbols.POLKADOT);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignRequest, { schema: unsignedTransactionSubstrate }, ProtocolSymbols_1.MainProtocolSymbols.KUSAMA);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignResponse, { schema: signedTransactionAeternity }, ProtocolSymbols_1.MainProtocolSymbols.AE);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignResponse, { schema: signedTransactionBitcoin }, ProtocolSymbols_1.MainProtocolSymbols.BTC);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignResponse, { schema: signedTransactionBitcoin }, ProtocolSymbols_1.MainProtocolSymbols.GRS);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignResponse, { schema: signedTransactionCosmos }, ProtocolSymbols_1.MainProtocolSymbols.COSMOS);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignResponse, { schema: signedTransactionEthereum }, ProtocolSymbols_1.MainProtocolSymbols.ETH);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignResponse, { schema: signedTransactionEthereum }, ProtocolSymbols_1.SubProtocolSymbols.ETH_ERC20);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignResponse, { schema: signedTransactionTezos }, ProtocolSymbols_1.MainProtocolSymbols.XTZ);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignResponse, { schema: signedTransactionTezos }, ProtocolSymbols_1.SubProtocolSymbols.XTZ_BTC);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignResponse, { schema: signedTransactionTezos }, ProtocolSymbols_1.SubProtocolSymbols.XTZ_KT);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignResponse, { schema: signedTransactionTezos }, ProtocolSymbols_1.SubProtocolSymbols.XTZ_USD);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignResponse, { schema: signedTransactionSubstrate }, ProtocolSymbols_1.MainProtocolSymbols.POLKADOT);
Serializer.addSchema(interfaces_1.IACMessageType.TransactionSignResponse, { schema: signedTransactionSubstrate }, ProtocolSymbols_1.MainProtocolSymbols.KUSAMA);
//# sourceMappingURL=serializer.js.map