"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var bs58check = require("../../dependencies/src/bs58check-2.1.2/index");
var rlp = require("../../dependencies/src/rlp-2.2.3/index");
var _1 = require(".");
var constants_1 = require("./constants");
var errors_1 = require("./errors");
var toBuffer_1 = require("./utils/toBuffer");
var wallet_sync_serializer_1 = require("./wallet-sync.serializer");
var SyncProtocolKeys;
(function (SyncProtocolKeys) {
    SyncProtocolKeys[SyncProtocolKeys["VERSION"] = 0] = "VERSION";
    SyncProtocolKeys[SyncProtocolKeys["TYPE"] = 1] = "TYPE";
    SyncProtocolKeys[SyncProtocolKeys["PROTOCOL"] = 2] = "PROTOCOL";
    SyncProtocolKeys[SyncProtocolKeys["PAYLOAD"] = 3] = "PAYLOAD";
})(SyncProtocolKeys = exports.SyncProtocolKeys || (exports.SyncProtocolKeys = {}));
var EncodedType;
(function (EncodedType) {
    EncodedType[EncodedType["UNSIGNED_TRANSACTION"] = 0] = "UNSIGNED_TRANSACTION";
    EncodedType[EncodedType["SIGNED_TRANSACTION"] = 1] = "SIGNED_TRANSACTION";
    EncodedType[EncodedType["WALLET_SYNC"] = 2] = "WALLET_SYNC";
})(EncodedType = exports.EncodedType || (exports.EncodedType = {}));
var SyncProtocolUtils = /** @class */ (function () {
    function SyncProtocolUtils() {
    }
    SyncProtocolUtils.prototype.serialize = function (deserializedSyncProtocol) {
        return __awaiter(this, void 0, void 0, function () {
            var version, type, protocol, typedPayload, untypedPayload, toRlpEncode, rlpEncoded;
            return __generator(this, function (_a) {
                version = toBuffer_1.toBuffer(constants_1.SERIALIZER_VERSION);
                type = toBuffer_1.toBuffer(deserializedSyncProtocol.type.toString());
                protocol = toBuffer_1.toBuffer(deserializedSyncProtocol.protocol);
                typedPayload = deserializedSyncProtocol.payload;
                switch (deserializedSyncProtocol.type) {
                    case EncodedType.UNSIGNED_TRANSACTION:
                        untypedPayload = _1.unsignedTransactionSerializerByProtocolIdentifier(deserializedSyncProtocol.protocol).serialize(typedPayload);
                        break;
                    case EncodedType.SIGNED_TRANSACTION:
                        untypedPayload = _1.signedTransactionSerializerByProtocolIdentifier(deserializedSyncProtocol.protocol).serialize(typedPayload);
                        break;
                    case EncodedType.WALLET_SYNC:
                        untypedPayload = new wallet_sync_serializer_1.WalletSerializer().serialize(typedPayload);
                        break;
                    default:
                        throw new errors_1.TypeNotSupported();
                }
                toRlpEncode = [];
                toRlpEncode[SyncProtocolKeys.VERSION] = version;
                toRlpEncode[SyncProtocolKeys.TYPE] = type;
                toRlpEncode[SyncProtocolKeys.PROTOCOL] = protocol;
                toRlpEncode[SyncProtocolKeys.PAYLOAD] = untypedPayload;
                rlpEncoded = rlp.encode(toRlpEncode);
                // as any is necessary due to https://github.com/ethereumjs/rlp/issues/35
                return [2 /*return*/, bs58check.encode(rlpEncoded)];
            });
        });
    };
    SyncProtocolUtils.prototype.deserialize = function (serializedSyncProtocol) {
        return __awaiter(this, void 0, void 0, function () {
            var base58Decoded, rlpDecodedTx, version, type, protocol, payload, typedPayload;
            return __generator(this, function (_a) {
                base58Decoded = bs58check.decode(serializedSyncProtocol);
                rlpDecodedTx = rlp.decode(base58Decoded);
                version = parseInt(rlpDecodedTx[SyncProtocolKeys.VERSION].toString(), 10);
                if (version !== constants_1.SERIALIZER_VERSION) {
                    throw new errors_1.SerializerVersionMismatch();
                }
                type = parseInt(rlpDecodedTx[SyncProtocolKeys.TYPE].toString(), 10);
                protocol = rlpDecodedTx[SyncProtocolKeys.PROTOCOL].toString() // TODO: Verify
                ;
                payload = rlpDecodedTx[SyncProtocolKeys.PAYLOAD];
                switch (type) {
                    case EncodedType.UNSIGNED_TRANSACTION:
                        typedPayload = _1.unsignedTransactionSerializerByProtocolIdentifier(protocol).deserialize(payload);
                        break;
                    case EncodedType.SIGNED_TRANSACTION:
                        typedPayload = _1.signedTransactionSerializerByProtocolIdentifier(protocol).deserialize(payload);
                        break;
                    case EncodedType.WALLET_SYNC:
                        typedPayload = new wallet_sync_serializer_1.WalletSerializer().deserialize(payload);
                        break;
                    default:
                        throw new errors_1.TypeNotSupported();
                }
                return [2 /*return*/, {
                        version: version,
                        type: type,
                        protocol: protocol,
                        payload: typedPayload
                    }];
            });
        });
    };
    return SyncProtocolUtils;
}());
exports.SyncProtocolUtils = SyncProtocolUtils;
//# sourceMappingURL=serializer.js.map