"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var wasm_crypto_1 = require("@polkadot/wasm-crypto");
var hex_1 = require("./hex");
function assertProperDerivationPath(path) {
    if (!['m', 'm/'].includes(path.slice(0, 2))) {
        throw new Error('Invalid derivation path');
    }
}
function getChainCode(value) {
    var chainCode = new Uint8Array(32);
    var index = parseInt(value, 10);
    var indexHex = hex_1.changeEndianness(hex_1.toHexStringRaw(index));
    chainCode.fill(0);
    chainCode.set(Buffer.from(indexHex, 'hex'));
    return chainCode;
}
function createDeriveJunction(value) {
    var isHard = ['h', "'"].includes(value.slice(-1));
    var code = isHard ? value.slice(0, -1) : value;
    return {
        chainCode: getChainCode(code),
        isHard: isHard
    };
}
function deriveFromPath(keyPair, path) {
    if (path.length == 0) {
        return Buffer.from(keyPair);
    }
    var deriveJunctions = path.split('/').map(createDeriveJunction);
    var derived = deriveJunctions.reduce(function (pair, junction) {
        var deriveKeypair = junction.isHard ? wasm_crypto_1.sr25519DeriveKeypairHard : wasm_crypto_1.sr25519DeriveKeypairSoft;
        return deriveKeypair(pair, junction.chainCode);
    }, keyPair);
    return Buffer.from(derived);
}
function createSr25519KeyPair(secret, derivationPath) {
    return __awaiter(this, void 0, void 0, function () {
        var seed, keyPair, derivedKeyPair;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    assertProperDerivationPath(derivationPath);
                    return [4 /*yield*/, wasm_crypto_1.waitReady()];
                case 1:
                    _a.sent();
                    seed = typeof secret === 'string' ? Buffer.from(hex_1.stripHexPrefix(secret), 'hex') : secret;
                    keyPair = wasm_crypto_1.sr25519KeypairFromSeed(seed.subarray(0, 32)) // 32-bit seed is required
                    ;
                    derivedKeyPair = deriveFromPath(keyPair, derivationPath.slice(2));
                    return [2 /*return*/, {
                            privateKey: derivedKeyPair.slice(0, 64),
                            publicKey: derivedKeyPair.slice(64)
                        }];
            }
        });
    });
}
exports.createSr25519KeyPair = createSr25519KeyPair;
//# sourceMappingURL=sr25519.js.map